﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ManagedDirectX;
using ManagedDirectX.XNAMath;
using ManagedDirectX.DXGI;
using ManagedDirectX.Direct3D11;
using ManagedDirectX.Direct3D11.Resource11;
using ManagedDirectX.Direct3DX11;
using D3D11Helper;

namespace Tutorial
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        DeviceManager manager;
        Object3D model;
        EffectShader eff;
        ManagedDirectX.Direct3D11.Resource11.Buffer constantBuffer;
        FontManager font;
        int n1, n2, n3;

        int lightType = 0;

        float angle = 0;

        public struct Data
        {
            public ManagedDirectX.XNAMath.Matrix TransformMatrix;
            public ManagedDirectX.XNAMath.Matrix World;
            public ManagedDirectX.XNAMath.Vector lightDirection;
            public ManagedDirectX.XNAMath.Vector lightPosition;
            public ManagedDirectX.XNAMath.Vector attenuation;
            public ManagedDirectX.XNAMath.Vector viewPosition;
            public ManagedDirectX.XNAMath.Vector diffuseColor;
            public ManagedDirectX.XNAMath.Vector specularColor;
            public float lightType;
            public float v1;
            public float v2;
            public float v3;
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            manager.Clear(0, 0, 1, 1);
            manager.ClearDepth(true, false, 1, 0);
            eff.Set(manager);
            SetMatrices();

            manager.SetDefaultState();

            model.SetBuffer(manager);
            model.Draw(manager);

            n1++;
            if (Environment.TickCount - n2 >= 1000)
            {
                n2 = Environment.TickCount;
                n3 = n1;
                n1 = 0;
            }

            font.DrawString("PRESS 1-5 FPS: " + n3.ToString(), 0, 0, ClientRectangle.Width, ClientRectangle.Height);

            switch (lightType)
            {
                case 0:
                    font.DrawString("Directional", 0, 30, ClientRectangle.Width, ClientRectangle.Height);
                    break;
                case 1:
                    font.DrawString("Point", 0, 30, ClientRectangle.Width, ClientRectangle.Height);
                    break;
                case 2:
                    font.DrawString("Specular Directional ", 0, 30, ClientRectangle.Width, ClientRectangle.Height);
                    break;
                case 3:
                    font.DrawString("Specular Point", 0, 30, ClientRectangle.Width, ClientRectangle.Height);
                    break;
                default:
                    font.DrawString("Off", 0, 30, ClientRectangle.Width, ClientRectangle.Height);
                    break;
            }



            manager.Present();
            this.Invalidate();

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true);
            manager = new D3D11Helper.DeviceManager(this.Handle, ClientRectangle.Width, ClientRectangle.Height);

            VersionNumber n = VersionNumber.Version4;
            eff = new EffectShader(Application.StartupPath + "\\Shader.hlsl", "VSMain", "PSMain", "", "", "", n, manager);

            model = Object3D.FromWavefrontOBJ(manager, eff, Application.StartupPath + "\\shapes.obj");

            constantBuffer = HelpFunction.CreateConstantBuffer<Data>(manager);

            font = new FontManager(manager);
        }


        private void SetMatrices()
        {

            ManagedDirectX.XNAMath.Matrix world = ManagedDirectX.XNAMath.Matrix.Identity();
            ManagedDirectX.XNAMath.Matrix view = ManagedDirectX.XNAMath.Matrix.LookAtLH(new Vector(30, 100, 100, 0), new Vector(), new Vector(0, 0, 1, 0));
            ManagedDirectX.XNAMath.Matrix proj = ManagedDirectX.XNAMath.Matrix.PerspectiveFovLH(3.14F / 3 - 0F, 1, 1, 1000);


            MappedSubResource sub = manager.DeviceContextPointer.Map(constantBuffer, 0, Map.WriteDiscard, MapFlags.None);


            angle = Environment.TickCount / 1000.0F;
            float x = (float)Math.Sin(angle);
            float y = (float)Math.Cos(angle);

            sub.WriteData<Data>(new Data()
            {
                TransformMatrix = world * view * proj,
                World = world,
                lightDirection = Vector3D.Normalize(new Vector(x, y, 1, 0)),
                lightPosition = new Vector(x * 20, y * 20, 30, 0),
                viewPosition = new Vector(30, 100, 100, 0),
                attenuation = new Vector(0.0001F, 0.01F, 0.0F, 0),
                diffuseColor = new Vector(1, 1, 1, 1),
                specularColor = new Vector(0.5F, 0.5F, 0, 0),
                lightType = lightType
            });
            manager.DeviceContextPointer.Unmap(constantBuffer, 0);
            manager.DeviceContextPointer.VSSetConstantBuffers(new ManagedDirectX.Direct3D11.Resource11.Buffer[] { constantBuffer });
            manager.DeviceContextPointer.PSSetConstantBuffers(new ManagedDirectX.Direct3D11.Resource11.Buffer[] { constantBuffer });

        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            manager.Resize(ClientRectangle.Width, ClientRectangle.Height);
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.D1:
                    lightType = 0;
                    break;
                case Keys.D2:
                    lightType = 1;
                    break;
                case Keys.D3:
                    lightType = 2;
                    break;
                case Keys.D4:
                    lightType = 3;
                    break;
                case Keys.D5:
                    lightType = 4;
                    break;
            }
        }
    }
}
