﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ManagedDirectX;
using ManagedDirectX.Direct3D11;
using ManagedDirectX.Direct3D11.Shader11;

namespace D3D11Helper
{
    /// <summary>
    /// Version number
    /// </summary>
    public enum VersionNumber
    {
        /// <summary>
        /// 4
        /// </summary>
        Version4,
        /// <summary>
        /// 5
        /// </summary>
        Version5
    }

    public class EffectShader
    {

        private enum ShaderType
        {
            VertexShader,
            PixelShader,
            GeometryShader,
            DomainShader,
            HullShader
        }



        public VertexShader VShader { get; private set; }

        public PixelShader PShader { get; private set; }

        public GeometryShader GShader { get; private set; }

        public DomainShader DShader { get; private set; }

        public HullShader HShader { get; private set; }

        public byte[] VertexShaderBuffer { get; private set; }


        public EffectShader(string filename, string vertexShaderEntry, string pixelShaderEntry, string geometryShaderEntry, string domainShaderEntry, string hullShaderEntry, VersionNumber number, DeviceManager manager)
        {

      
            string code = System.IO.File.ReadAllText(filename);

            if (!string.IsNullOrEmpty(vertexShaderEntry))
            {
                byte[] vsCode;
                VShader = (VertexShader)Create(code, number, vertexShaderEntry, manager, ShaderType.VertexShader, out vsCode);
                VertexShaderBuffer = vsCode;
            }

            if (!string.IsNullOrEmpty(pixelShaderEntry))
                PShader = (PixelShader)Create(code, number, pixelShaderEntry, manager, ShaderType.PixelShader);

            if (!string.IsNullOrEmpty(geometryShaderEntry))
                GShader = (GeometryShader)Create(code, number, geometryShaderEntry, manager, ShaderType.GeometryShader);

            if (!string.IsNullOrEmpty(hullShaderEntry))
                HShader = (HullShader)Create(code, number, hullShaderEntry, manager, ShaderType.GeometryShader);

            if (!string.IsNullOrEmpty(domainShaderEntry))
                DShader = (DomainShader)Create(code, number, domainShaderEntry, manager, ShaderType.GeometryShader);


        }

        public void Set(DeviceManager manager)
        {
            manager.DeviceContextPointer.VSSetShader(VShader);
            manager.DeviceContextPointer.GSSetShader(GShader);
            manager.DeviceContextPointer.PSSetShader(PShader);
            manager.DeviceContextPointer.HSSetShader(HShader);
            manager.DeviceContextPointer.DSSetShader(DShader);
        }


        private BaseShader Create(string code, VersionNumber number, string entryPoint, DeviceManager manager, ShaderType type)
        {

            byte[] byteCode;
            return Create(code, number, entryPoint, manager, type, out byteCode);
        }

        private BaseShader Create(string code, VersionNumber number, string entryPoint, DeviceManager manager, ShaderType type, out byte[] byteCode)
        {
            string profile = (number == VersionNumber.Version4) ? "4" : "5";
            switch (type)
            {
                case ShaderType.VertexShader:
                    profile = string.Format("vs_{0}_0", profile);
                    break;
                case ShaderType.PixelShader:
                    profile = string.Format("ps_{0}_0", profile);
                    break;
                case ShaderType.GeometryShader:
                    profile = string.Format("gs_{0}_0", profile);
                    break;
                case ShaderType.DomainShader:
                    profile = string.Format("ds_{0}_0", profile);
                    break;
                case ShaderType.HullShader:
                    profile = string.Format("hs_{0}_0", profile);
                    break;
                default:
                    break;
            }

            byteCode = BaseShader.CompileShader(code, entryPoint, profile, HLSLFlags.None);

            switch (type)
            {
                case ShaderType.VertexShader:
                    return manager.DevicePointer.CreateVertexShader(byteCode);
                case ShaderType.PixelShader:
                    return manager.DevicePointer.CreatePixelShader(byteCode);
                case ShaderType.GeometryShader:
                    return manager.DevicePointer.CreateGeometryShader(byteCode);
                case ShaderType.HullShader:
                    return manager.DevicePointer.CreateHullShader(byteCode);
                case ShaderType.DomainShader:
                    return manager.DevicePointer.CreateDomainShader(byteCode);
            }

            throw new Exception();

        }
    }
}
