﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ManagedDirectX;
using ManagedDirectX.Direct3D11.Resource11;
using ManagedDirectX.XNAMath;
using ManagedDirectX.Direct3D11;
using Buffer11 = ManagedDirectX.Direct3D11.Resource11.Buffer;


namespace D3D11Helper
{
    /// <summary>
    /// Font manager
    /// </summary>
    public class FontManager
    {
        private const int glyphSize = 256;
        private InputLayout layout;
        private Vector[] fontDescription = new Vector[100];
        private Vector[] currentDesc = new Vector[800];
        private ShaderResourceView fontTexture;
        private int textLength;
        private DeviceManager manager;
        private EffectShader effect;
        private Buffer11 buffer;
        private Buffer11 vertexBuffer;

        public FontManager(DeviceManager manager)
        {

            this.manager = manager;

            textLength = 0;

            fontDescription[0] = new Vector(0.00F, 0.00F, 5.33F, 19.88F);
            fontDescription[1] = new Vector(22.00F, 0.00F, 9.91F, 19.88F);
            fontDescription[2] = new Vector(44.00F, 0.00F, 11.18F, 19.88F);
            fontDescription[3] = new Vector(66.00F, 0.00F, 14.50F, 19.88F);
            fontDescription[4] = new Vector(88.00F, 0.00F, 14.50F, 19.88F);
            fontDescription[5] = new Vector(110.00F, 0.00F, 19.99F, 19.88F);
            fontDescription[6] = new Vector(132.00F, 0.00F, 19.99F, 19.88F);
            fontDescription[7] = new Vector(154.00F, 0.00F, 19.99F, 19.88F);
            fontDescription[8] = new Vector(176.00F, 0.00F, 19.99F, 19.88F);
            fontDescription[9] = new Vector(198.00F, 0.00F, 19.99F, 19.88F);
            fontDescription[10] = new Vector(0.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[11] = new Vector(22.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[12] = new Vector(44.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[13] = new Vector(66.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[14] = new Vector(88.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[15] = new Vector(110.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[16] = new Vector(132.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[17] = new Vector(154.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[18] = new Vector(176.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[19] = new Vector(198.00F, 22.00F, 19.99F, 19.88F);
            fontDescription[20] = new Vector(0.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[21] = new Vector(22.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[22] = new Vector(44.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[23] = new Vector(66.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[24] = new Vector(88.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[25] = new Vector(110.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[26] = new Vector(132.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[27] = new Vector(154.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[28] = new Vector(176.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[29] = new Vector(198.00F, 44.00F, 19.99F, 19.88F);
            fontDescription[30] = new Vector(0.00F, 66.00F, 19.99F, 19.88F);
            fontDescription[31] = new Vector(22.00F, 66.00F, 19.99F, 19.88F);
            fontDescription[32] = new Vector(44.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[33] = new Vector(66.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[34] = new Vector(88.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[35] = new Vector(110.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[36] = new Vector(132.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[37] = new Vector(154.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[38] = new Vector(176.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[39] = new Vector(198.00F, 66.00F, 22.06F, 19.88F);
            fontDescription[40] = new Vector(0.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[41] = new Vector(22.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[42] = new Vector(44.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[43] = new Vector(66.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[44] = new Vector(88.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[45] = new Vector(110.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[46] = new Vector(132.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[47] = new Vector(154.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[48] = new Vector(176.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[49] = new Vector(198.00F, 88.00F, 22.06F, 19.88F);
            fontDescription[50] = new Vector(0.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[51] = new Vector(22.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[52] = new Vector(44.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[53] = new Vector(66.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[54] = new Vector(88.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[55] = new Vector(110.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[56] = new Vector(132.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[57] = new Vector(154.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[58] = new Vector(176.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[59] = new Vector(198.00F, 110.00F, 22.06F, 19.88F);
            fontDescription[60] = new Vector(0.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[61] = new Vector(22.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[62] = new Vector(44.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[63] = new Vector(66.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[64] = new Vector(88.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[65] = new Vector(110.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[66] = new Vector(132.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[67] = new Vector(154.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[68] = new Vector(176.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[69] = new Vector(198.00F, 132.00F, 22.06F, 19.88F);
            fontDescription[70] = new Vector(0.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[71] = new Vector(22.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[72] = new Vector(44.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[73] = new Vector(66.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[74] = new Vector(88.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[75] = new Vector(110.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[76] = new Vector(132.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[77] = new Vector(154.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[78] = new Vector(176.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[79] = new Vector(198.00F, 154.00F, 22.06F, 19.88F);
            fontDescription[80] = new Vector(0.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[81] = new Vector(22.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[82] = new Vector(44.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[83] = new Vector(66.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[84] = new Vector(88.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[85] = new Vector(110.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[86] = new Vector(132.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[87] = new Vector(154.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[88] = new Vector(176.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[89] = new Vector(198.00F, 176.00F, 22.06F, 19.88F);
            fontDescription[90] = new Vector(0.00F, 198.00F, 22.06F, 19.88F);
            fontDescription[91] = new Vector(22.00F, 198.00F, 22.06F, 19.88F);
            fontDescription[92] = new Vector(44.00F, 198.00F, 22.06F, 19.88F);
            fontDescription[93] = new Vector(66.00F, 198.00F, 22.06F, 19.88F);
            fontDescription[94] = new Vector(88.00F, 198.00F, 22.06F, 19.88F);



            effect = new EffectShader(System.Windows.Forms.Application.StartupPath + "\\Framework\\FontShader.hlsl", "VSMain", "PSMain", "", "", "", VersionNumber.Version4, manager);


            //creazione input layout
            InputElementDescription[] layoutDesc =
	        {
		        new InputElementDescription( "POSITION", 0,ManagedDirectX.DXGI.Format.R32_FLOAT , 0, 0,InputClassification.VertexData , 0 ) 
	        };

            layout = manager.DevicePointer.CreateInputLayout(layoutDesc, effect.VertexShaderBuffer);

            float[] temp = new float[1200];
            for (int i = 0; i < 200; i++)
            {
                temp[i * 6] = (i * 4.0F);
                temp[i * 6 + 1] = (i * 4.0F + 1);
                temp[i * 6 + 2] = (i * 4.0F + 2);

                temp[i * 6 + 3] = (i * 4.0F);
                temp[i * 6 + 4] = (i * 4.0F + 2);
                temp[i * 6 + 5] = (i * 4.0F + 3);
            }

            //creazione vertex buffer
            BufferDescription bd = new BufferDescription();
            bd.Usage = Usage.Default;
            bd.ByteWidth = 4 * 6 * 200;
            bd.BindFlags = BindingFlag.VertexBuffer;
            bd.CPUAccessFlags = 0;
            bd.MiscFlags = 0;
            SubResourceData InitData = new SubResourceData(ManagedDirectX.MathUtility.ArrayToByte<float>(temp), 0, 0);
            vertexBuffer = manager.DevicePointer.CreateBuffer(bd, InitData);


            buffer = HelpFunction.CreateConstantBuffer(manager, 800 * 16);

            fontTexture = ShaderResourceView.CreateShaderResourceViewFromFile(manager.DevicePointer, System.Windows.Forms.Application.StartupPath + "\\Framework\\font.png");

        }

        public void DrawString(string text, int x, int y, int width, int height)
        {
            SetText(text, x, y, width, height);
            Draw();
        }

        public void SetText(string text, int x, int y, int width, int height)
        {
            textLength = text.Length;

            float tempV = 0;
            int k = 0;
            for (int i = 0; i < textLength; i++)
            {
                int n = (int)text[i];
                float vX = fontDescription[n - 32].X;
                float vY = fontDescription[n - 32].Y;
                float vZ = fontDescription[n - 32].Z;
                float vW = fontDescription[n - 32].W;

                currentDesc[k] = new Vector(
                    ((x + tempV) / (float)width) * 2.0F - 1.0F,
                    (-y / (float)height) * 2.0F + 1.0F,
                    vX / glyphSize, vY / glyphSize);
                k++;

                currentDesc[k] = new Vector(
                    ((x + vZ + tempV) / (float)width) * 2.0F - 1.0F,
                    (-y / (float)height) * 2.0F + 1.0F,
                    (vX + vZ) / glyphSize, vY / glyphSize);
                k++;

                currentDesc[k] = new Vector(
                    ((x + vZ + tempV) / (float)width) * 2.0F - 1.0F,
                    ((-y - vW) / (float)height) * 2.0F + 1.0F,
                    (vX + vZ) / glyphSize, (vY + vW) / glyphSize);
                k++;

                currentDesc[k] = new Vector(
                    ((x + tempV) / (float)width) * 2.0F - 1.0F,
                    ((-y - vW) / (float)height) * 2.0F + 1.0F,
                    vX / glyphSize, (vY + vW) / glyphSize);
                k++;

                tempV += vZ;
            }

            MappedSubResource sub = manager.DeviceContextPointer.Map(buffer, 0, Map.WriteDiscard, MapFlags.None);
            sub.WriteData1D<Vector>(currentDesc);
            manager.DeviceContextPointer.Unmap(buffer, 0);

        }

        public void Draw()
        {
            if (textLength == 0)
                return;

            effect.Set(manager);
            Buffer11[] buffers = { buffer };
            ShaderResourceView[] views = { fontTexture };

            manager.DeviceContextPointer.VSSetConstantBuffers(buffers);
            manager.DeviceContextPointer.PSSetShaderResources(views);
            manager.DeviceContextPointer.IASetPrimitiveTopology(PrimitiveTopology.TriangleList);
            manager.DeviceContextPointer.IASetInputLayout(layout);
            manager.DeviceContextPointer.IASetVertexBuffers(vertexBuffer, 4);
            manager.DeviceContextPointer.Draw(textLength * 6, 0);

        }


    }
}
