﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ManagedDirectX;

namespace D3D11Helper
{

    /// <summary>
    /// Position vertex
    /// </summary>
    public struct PositionOnly
    {
        /// <summary>
        /// Position
        /// </summary>
        public Vector3 Position;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="z">Z</param>
        public PositionOnly(float x, float y, float z)
        {
            Position = new Vector3(x, y, z);
        }
    }

    /// <summary>
    /// Position-normal vertex
    /// </summary>
    public struct PositionNormal
    {
        /// <summary>
        /// Position
        /// </summary>
        public Vector3 Position;

        /// <summary>
        /// Normal
        /// </summary>
        public Vector3 Normal;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="z">Z</param>
        /// <param name="nx">Normal x</param>
        /// <param name="ny">Normal y</param>
        /// <param name="nz">Normal z</param>
        public PositionNormal(float x, float y, float z, float nx, float ny, float nz)
        {
            Position = new Vector3(x, y, z);
            Normal = new Vector3(nx, ny, nz);
        }
    }

    /// <summary>
    /// Position/Normal/Texture vertex
    /// </summary>
    public struct Vertex
    {
        /// <summary>
        /// Position
        /// </summary>
        public Vector3 Position;

        /// <summary>
        /// Normal
        /// </summary>
        public Vector3 Normal;

        /// <summary>
        /// Texture
        /// </summary>
        public Vector2 Texture;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="z">Z</param>
        /// <param name="nx">Nx</param>
        /// <param name="ny">Ny</param>
        /// <param name="nz">Nz</param>
        /// <param name="tu">Tu</param>
        /// <param name="tv">Tv</param>
        public Vertex(float x, float y, float z, float nx, float ny, float nz, float tu, float tv)
        {
            Position = new Vector3(x, y, z);
            Normal = new Vector3(nx, ny, nz);
            Texture = new Vector2(tu, tv);
        }
    }

}
