
cbuffer data :register(c0)
{
	float4x4 transform;
	float4x4 world;
	float4 lightDirection;
	float4 lightPosition;
	float4 attenuation;
	float4 viewPosition;
	float4 diffuseColor;
	float4 specularColor;
	float lightType;
};


Texture2D    diffuse     : register( t0 );

SamplerState samplerDiffuse : register( s0 )
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};


struct VS_INPUT
{
    float4 position                        : POSITION;
    float2 textureC                        : TEXCOORD0;
    float3 normal                          : NORMAL;
};

struct VS_OUTPUT
{
    float4 position : SV_POSITION;
    float3 normal: NORMAL;
	float3 posWorld:POSITION_W;
    float2 textureC : TEXCOORD0;
};


VS_OUTPUT VSMain( VS_INPUT input )
{
    VS_OUTPUT Output;

    Output.position = mul(transform,input.position);
    Output.textureC=input.textureC ;
    Output.normal=mul(world,input.normal);
	Output.posWorld=mul(world,input.position);
    return Output;
}

float4 PSMain( VS_OUTPUT input ) : SV_TARGET
{
	float4 color= diffuse.Sample(samplerDiffuse,float2(input.textureC.x,input.textureC.y));
	float3 N=normalize(input.normal);
	switch(lightType)
	{
		case 0:
			{
				return saturate(dot(lightDirection.xyz,N)) * color*diffuseColor;
			}
		case 1:
			{
				float3 L=normalize(lightPosition.xyz-input.posWorld);
				float d=length(input.posWorld-lightPosition.xyz);
				float att=1.0F/ (d*d*attenuation.x+d*attenuation.y+attenuation.z);
				return saturate(dot(L.xyz,N)) * att * color*diffuseColor;
			}
		case 2:
			{
				float4 diffuse = saturate(dot(lightDirection.xyz,N)) * color;
				float3 halfway = normalize(lightDirection.xyz + normalize(viewPosition));
				float4 specular = pow(saturate(dot(N, halfway)), 20);
				return diffuse*diffuseColor+specular*specularColor;
			}
		case 3:
			{
				float3 L=normalize(lightPosition.xyz-input.posWorld);
				float d=length(input.posWorld-lightPosition.xyz);
				float att=1.0F/ (d*d*attenuation.x+d*attenuation.y+attenuation.z);
				float4 diffuse= saturate(dot(L.xyz,N)) * att * color;
				
				float3 halfway = normalize(L + normalize(viewPosition));
				float4 specular = pow(saturate(dot(N, halfway)), 20);
				return diffuse*diffuseColor+specular*specularColor;
			}
		default:
			return color;
	}
	
	return saturate(dot(lightDirection.xyz,input.normal)) * color;
}